/******************************************************************************
 *   @file   adf4156.c
 *   @brief  Implementation of ADF4156 Driver for NIOS processor.
 *   @author ATofan (alexandru.tofan@analog.com)
 *
 *******************************************************************************
 * Copyright 2011(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *   SVN Revision: 861
 ******************************************************************************/
#define ADF4156_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "adf4156.h"
#include <altera_avalon_pio_regs.h> // PIO access macros
#include <altera_avalon_spi.h>      // SPI functions
#include <unistd.h>                 // usleep();
#include <system.h>
#if ( USE_ADF4156 == 1 )

/*****************************************************************************/
/* Private variables                                                         */
/*****************************************************************************/
unsigned char   txBuffer[4]     = {0,};     // Tx SPI Buffer.
unsigned char   rxBuffer[4]     = {0,};     // Rx SPI Buffer.
char adf4156ModR0Prev           = 0;
char adf4156ModR1Prev           = 0;
char adf4156ModR2Prev           = 0;
char adf4156ModR3Prev           = 0;
char adf4156ModR4Prev           = 0;
char adf4156ModAllPrev          = 0;

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
// R0
volatile char* adf4156MuxOut        = &ADF4156_Data[0].chData;
volatile int*  adf4156IntVal        = &ADF4156_Data[1].iData;
volatile int*  adf4156FracVal       = &ADF4156_Data[2].iData;
// R1
volatile int*  adf4156PhaseVal      = &ADF4156_Data[3].iData;
// R2
volatile char* adf4156Noise         = &ADF4156_Data[4].chData;
volatile char* adf4156CsrEn         = &ADF4156_Data[5].chData;
volatile char* adf4156CurrSet       = &ADF4156_Data[6].chData;
volatile char* adf4156Prescaler     = &ADF4156_Data[7].chData;
volatile char* adf4156RDiv2         = &ADF4156_Data[8].chData;
volatile char* adf4156RefDbl        = &ADF4156_Data[9].chData;
volatile int*  adf4156RCnt          = &ADF4156_Data[10].iData;
volatile int*  adf4156Modulus       = &ADF4156_Data[11].iData;
// R3
volatile char* adf4156SigDelRst     = &ADF4156_Data[12].chData;
volatile char* adf4156Ldp           = &ADF4156_Data[13].chData;
volatile char* adf4156PdPol         = &ADF4156_Data[14].chData;
volatile char* adf4156CpZ           = &ADF4156_Data[15].chData;
volatile char* adf4156CntRst        = &ADF4156_Data[16].chData;
// R4
volatile char* adf4156ClkDivMode    = &ADF4156_Data[17].chData;
volatile int*  adf4156ClkDivVal     = &ADF4156_Data[18].iData;
// SPI ERROR
volatile char* adf4156SPIErr        = &ADF4156_Data[19].chData;
// Modify buttons
volatile char* adf4156ModR0         = &ADF4156_Data[20].chData;
volatile char* adf4156ModR1         = &ADF4156_Data[21].chData;
volatile char* adf4156ModR2         = &ADF4156_Data[22].chData;
volatile char* adf4156ModR3         = &ADF4156_Data[23].chData;
volatile char* adf4156ModR4         = &ADF4156_Data[24].chData;
volatile char* adf4156ModAll        = &ADF4156_Data[25].chData;

/******************************************************************************
 * @brief Initialize SPI and the ADF4156 Chip
 *
 * @param None.
 *
 * @return Always returns SUCCESS
 ******************************************************************************/
int ADF4156_Init( void )
{
    int cfgValue = 0;
    // setup SPI interface
    SetCtrl(ADF4156_SPI_CFG);

    IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, (GPIO_LE|GPIO_CE));
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_CE);
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LE);

    // R4
    cfgValue = ADF4156_CLK_DIV_OFF      |
               ADF4156_CLK_DIV_VAL(1)   |
               ADF4156_R4_CTRL;
    ADF4156_Set(cfgValue);

    // R3
    cfgValue = ADF4156_SIG_DEL_RST(1)   |
               ADF4156_LDP(0)           |
               ADF4156_PD_POL(1)        |
               ADF4156_CP_Z(0)          |
               ADF4156_CNT_RST(0)       |
               ADF4156_R3_CTRL;
    ADF4156_Set(cfgValue);

    // R2
    cfgValue = ADF4156_LOW_NOISE        |
               ADF4156_CSR_EN(1)        |
               ADF4156_CURR_SET(0)      |
               ADF4156_PRESCALER(0)     |
               ADF4156_RDIV2(1)         |
               ADF4156_REF_DBL(0)       |
               ADF4156_R_CNT(1)         |
               ADF4156_MOD_WORD(0)      |
               ADF4156_R2_CTRL;
    ADF4156_Set(cfgValue);

    // R1
    cfgValue = ADF4156_PHASE_VAL(1)     |
               ADF4156_R1_CTRL;
    ADF4156_Set(cfgValue);

    // R0
    cfgValue = ADF4156_MUXOUT_SDO       |
               ADF4156_INT_VAL(1)       |
               ADF4156_FRAC_VAL(1)      |
               ADF4156_R0_CTRL;
    ADF4156_Set(cfgValue);

    return(SUCCESS);
}

/******************************************************************************
 * @brief Send SPI Data
 *
 * @param value
 *
 * @return SUCCESS if the returned value is the same with the one sent
 *         FAIL otherwise
 ******************************************************************************/
int ADF4156_Set( volatile int value )
{
    int validation = 0;
    txBuffer[0] = (unsigned char)((value & 0xFF000000) >> 24);
    txBuffer[1] = (unsigned char)((value & 0x00FF0000) >> 16);
    txBuffer[2] = (unsigned char)((value & 0x0000FF00) >> 8);
    txBuffer[3] = (unsigned char)(value & 0x000000FF);

    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LE);

    alt_avalon_spi_command(ADF4156_SPI, 0, 4, txBuffer, 0, rxBuffer,0);

    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_LE);
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LE);

    alt_avalon_spi_command(ADF4156_SPI, 0, 0, txBuffer, 4, rxBuffer,0);

    validation += (int) ( rxBuffer[0] << 24 );
    validation += (int) ( rxBuffer[1] << 16 );
    validation += (int) ( rxBuffer[2] << 8 );
    validation += (int)rxBuffer[3];

    if (validation != value)
    {
        *adf4156SPIErr = 1;
        return(FAIL);
    }
    else
    {
        *adf4156SPIErr = 0;
    }
    return(SUCCESS);
}

/******************************************************************************
 * @brief Update the devices monitored and controlled parameters
 *
 * @param None
 *
 * @return Always returns SUCCESS
 ******************************************************************************/
int ADF4156_UpdateParameters(void)
{
    if(*adf4156ModAll != adf4156ModAllPrev)
    {
        ADF4156_SetR4();
        ADF4156_SetR3();
        ADF4156_SetR2();
        ADF4156_SetR1();
        ADF4156_SetR0();
        adf4156ModAllPrev = *adf4156ModAll;
    }
    return(SUCCESS);
}

/******************************************************************************
 * @brief Set R0 register
 *
 * @param None
 *
 * @return Always returns SUCCESS
 ******************************************************************************/
int ADF4156_SetR0(void)
{
    int cfgValue = 0;

    // R0
    switch(*adf4156MuxOut)
    {
        case 0x00:
            cfgValue |= ADF4156_MUXOUT_Z;
            break;
        case 0x01:
            cfgValue |= ADF4156_MUXOUT_DVdd;
            break;
        case 0x02:
            cfgValue |= ADF4156_MUXOUT_DGND;
            break;
        case 0x03:
            cfgValue |= ADF4156_MUXOUT_RDIV;
            break;
        case 0x04:
            cfgValue |= ADF4156_MUXOUT_NDIV;
            break;
        case 0x05:
            cfgValue |= ADF4156_MUXOUT_ALOCK;
            break;
        case 0x06:
            cfgValue |= ADF4156_MUXOUT_DLOCK;
            break;
        case 0x07:
            cfgValue |= ADF4156_MUXOUT_SDO;
            break;
        case 0x08:
            cfgValue |= ADF4156_MUXOUT_CLKDIV;
            break;
        case 0x09:
            cfgValue |= ADF4156_MUXOUT_FLS;
            break;
        case 0x0A:
            cfgValue |= ADF4156_MUXOUT_RDIV2;
            break;
        case 0x0B:
            cfgValue |= ADF4156_MUXOUT_NDIV2;
            break;
        default:
            break;
    }

    cfgValue |= ADF4156_INT_VAL(*adf4156IntVal);
    cfgValue |= ADF4156_FRAC_VAL(*adf4156FracVal);
    cfgValue |= ADF4156_R0_CTRL;

    ADF4156_Set(cfgValue);

    return(SUCCESS);
}

/******************************************************************************
 * @brief Set R1 register
 *
 * @param None
 *
 * @return Always returns SUCCESS
 ******************************************************************************/
int ADF4156_SetR1(void)
{
    int cfgValue = 0;

    cfgValue |= ADF4156_PHASE_VAL(*adf4156PhaseVal);
    cfgValue |= ADF4156_R1_CTRL;

    ADF4156_Set(cfgValue);

    return(SUCCESS);
}

/******************************************************************************
 * @brief Set R2 register
 *
 * @param None
 *
 * @return Always returns SUCCESS
 ******************************************************************************/
int ADF4156_SetR2(void)
{
    int cfgValue = 0;

    switch(*adf4156Noise)
    {
        case 0:
            cfgValue |= ADF4156_LOW_NOISE;
            break;
        case 1:
            cfgValue |= ADF4156_LOW_SPUR;
            break;
        default:
            break;
    }

    cfgValue |= ADF4156_CSR_EN(*adf4156CsrEn);
    cfgValue |= ADF4156_CURR_SET(*adf4156CurrSet);
    cfgValue |= ADF4156_PRESCALER(*adf4156Prescaler);
    cfgValue |= ADF4156_RDIV2(*adf4156RDiv2);
    cfgValue |= ADF4156_REF_DBL(*adf4156RefDbl);
    cfgValue |= ADF4156_R_CNT(*adf4156RCnt);
    cfgValue |= ADF4156_MOD_WORD(*adf4156Modulus);
    cfgValue |= ADF4156_R2_CTRL;

    ADF4156_Set(cfgValue);

    return(SUCCESS);
}

/******************************************************************************
 * @brief Set R3 register
 *
 * @param None
 *
 * @return Always returns SUCCESS
 ******************************************************************************/
int ADF4156_SetR3(void)
{
    int cfgValue = 0;

    cfgValue |= ADF4156_SIG_DEL_RST(*adf4156SigDelRst);
    cfgValue |= ADF4156_LDP(*adf4156Ldp);
    cfgValue |= ADF4156_PD_POL(*adf4156PdPol);
    cfgValue |= ADF4156_CP_Z(*adf4156CpZ);
    cfgValue |= ADF4156_CNT_RST(*adf4156CntRst);
    cfgValue |= ADF4156_R3_CTRL;

    ADF4156_Set(cfgValue);

    return(SUCCESS);
}

/******************************************************************************
 * @brief Set R4 register
 *
 * @param None
 *
 * @return Always returns SUCCESS
 ******************************************************************************/
int ADF4156_SetR4(void)
{
    int cfgValue = 0;

    switch(*adf4156ClkDivMode)
    {
        case 0:
            cfgValue |= ADF4156_CLK_DIV_OFF;
            break;
        case 1:
            cfgValue |= ADF4156_CLK_FAST_LOCK;
            break;
        case 2:
            cfgValue |= ADF4156_CLK_RESYNC_EN;
            break;
        default:
            break;
    }

    cfgValue |= ADF4156_CLK_DIV_VAL(*adf4156ClkDivVal);
    cfgValue |= ADF4156_R4_CTRL;

    ADF4156_Set(cfgValue);

    return(SUCCESS);
}
#endif
