/******************************************************************************
*   @file   adf4156.h
*   @brief  Header file of ADF4156 Driver for NIOS processor.
 *   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 861
******************************************************************************/
#ifndef _ADF4156_H_
#define _ADF4156_H_
#ifdef ADF4156_GLOBALS
    #define ADF4156_EXT extern volatile
#else
    #define ADF4156_EXT volatile
#endif

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "adi.h"
#if (USE_ADF4156 == 1)

/*****************************************************************************/
/* ADF4156                                                                    */
/*****************************************************************************/
// FRAC/INT Register R0
// MUXOUT Control
#define ADF4156_MUXOUT_Z        (0x00 << 27)
#define ADF4156_MUXOUT_DVdd     (0x01 << 27)
#define ADF4156_MUXOUT_DGND     (0x02 << 27)
#define ADF4156_MUXOUT_RDIV     (0x03 << 27)
#define ADF4156_MUXOUT_NDIV     (0x04 << 27)
#define ADF4156_MUXOUT_ALOCK    (0x05 << 27)
#define ADF4156_MUXOUT_DLOCK    (0x06 << 27)
#define ADF4156_MUXOUT_SDO      (0x07 << 27)
#define ADF4156_MUXOUT_CLKDIV   (0x0A << 27)
#define ADF4156_MUXOUT_FLS      (0x0C << 27)
#define ADF4156_MUXOUT_RDIV2    (0x0D << 27)
#define ADF4156_MUXOUT_NDIV2    (0x0E << 27)
// 12-Bit Integer Value INT
#define ADF4156_INT_VAL(x)      (x << 15)
// 12-Bit Fractional Value FRAC
#define ADF4156_FRAC_VAL(x)     (x << 3)
// Control bits
#define ADF4156_R0_CTRL         0x00

// PHASE Register R1
#define ADF4156_PHASE_VAL(x)    (x << 3)
#define ADF4156_R1_CTRL         0x01

// MOD/R Register R2
// Noise Mode
#define ADF4156_LOW_NOISE       (0x00 << 29)
#define ADF4156_LOW_SPUR        (0x03 << 29)
// CSR EN
#define ADF4156_CSR_EN(x)       (x << 28)
// Current setting
#define ADF4156_CURR_SET(x)     (x << 24)
// Prescaler
#define ADF4156_PRESCALER(x)    (x << 22)
// RDIV2
#define ADF4156_RDIV2(x)        (x << 21)
// Reference Doubler
#define ADF4156_REF_DBL(x)      (x << 20)
// 5-Bit R-Counter
#define ADF4156_R_CNT(x)        (x << 15)
// 12-Bit Modulus Word
#define ADF4156_MOD_WORD(x)     (x << 3)
// Control bits
#define ADF4156_R2_CTRL         0x02

// Function Register R3
// Sigma Delta Reset
#define ADF4156_SIG_DEL_RST(x)  (x << 14)
// LDP
#define ADF4156_LDP(x)          (x << 7)
// PD Polarity
#define ADF4156_PD_POL(x)       (x << 6)
// CP Three State
#define ADF4156_CP_Z(x)         (x << 4)
// Counter Reset
#define ADF4156_CNT_RST(x)      (x << 3)
// Control bits
#define ADF4156_R3_CTRL         0x03

// CLK DIV Register R4
// CLK DIV Mode
#define ADF4156_CLK_DIV_OFF     (0x00 << 19)
#define ADF4156_CLK_FAST_LOCK   (0x01 << 19)
#define ADF4156_CLK_RESYNC_EN   (0x02 << 19)
// CLK DIV VALUE
#define ADF4156_CLK_DIV_VAL(x)  (x << 7)
// Control bits
#define ADF4156_R4_CTRL         0x04

// GPIO
#define GPIO_LE                         (1 << 0)
#define GPIO_CE                         (1 << 2)

/*****************************************************************************/
/* SPI Configuration                                                         */
/*****************************************************************************/
#define ADF4156_SPI      SPI_0_P0_BASE
#define ADF4156_SPI_CFG  (SPI0_ena)

/*****************************************************************************/
/* Monitored variables declarations                                          */
/*****************************************************************************/
ADF4156_EXT uCProbeDataUnion ADF4156_Data[26];	//used to store the values for uC-Probe communication
#endif //(USE_ADF4156 == 1)

/*****************************************************************************/
/*  Functions Prototypes                                                     */
/*****************************************************************************/
int ADF4156_Init(void);                 // Initialize SPI and the ADF4156 Chip
int ADF4156_UpdateParameters(void);   	// Update the devices monitored and controlled parameters
int ADF4156_Set(volatile int value);	// Send SPI data
int ADF4156_SetR0(void);              	// Set R0 register
int ADF4156_SetR1(void);               	// Set R1 register
int ADF4156_SetR2(void);              	// Set R2 register
int ADF4156_SetR3(void);              	// Set R3 register
int ADF4156_SetR4(void);             	// Set R4 register
#endif /* _ADF4156_H_ */
